Imports System.ComponentModel


Public Class FeatureSummaryPropertyDescriptor
    Inherits PropertyDescriptor
    Public Const STR_PropertyName As String = "% of average"

    Public Sub New(ByVal featureList As BindingList(Of Feature), ByVal voteList As BindingList(Of Vote))
        mybase.New(STR_PropertyName, nothing)
        _featureList = featureList
        _voteList = voteList
        AddHandler _voteList.ListChanged, New ListChangedEventHandler(AddressOf voteList_ListChanged)
    End Sub

    Private _featureList As BindingList(Of Feature)
    Public ReadOnly Property FeatureList() As BindingList(Of Feature)
        Get
            Return _featureList
        End Get
    End Property

    Private _voteList As BindingList(Of Vote)
    Public ReadOnly Property VoteList() As BindingList(Of Vote)
        Get
            Return _voteList
        End Get
    End Property

    Private Sub voteList_ListChanged(ByVal sender As Object, ByVal e As ListChangedEventArgs)
        If e.ListChangedType = ListChangedType.ItemChanged Then
            PushChangeToAllFeatures()
        End If
    End Sub

    Private Sub PushChangeToAllFeatures()
        For Each _feature As Feature In FeatureList
            _feature.RaisePropertyChanged(STR_PropertyName)
        Next
    End Sub

    Public Overrides Function CanResetValue(ByVal component As Object) As Boolean
        Return False
    End Function

    Public Overrides ReadOnly Property ComponentType() As System.Type
        Get
            Return GetType(Feature)
        End Get
    End Property

    Public Overrides Function GetValue(ByVal component As Object) As Object
        Dim _feature As Feature = CType(component, Feature)
        Return CalcPercentOfAverage(_feature)
    End Function

    Private Function CalcPercentOfAverage(ByVal _feature As Feature) As Double
        Dim averageInfos As Dictionary(Of Feature, FeatureInfo) = CalcYearAveragePerFeature()
        If averageInfos.ContainsKey(_feature) Then
            Dim thisFeatureAverage As Double = averageInfos(_feature).Average
            Dim overallAverage As Double = 0
            For Each info As FeatureInfo In averageInfos.Values
                overallAverage += info.Average
            Next
            overallAverage /= averageInfos.Values.Count
            Return thisFeatureAverage / overallAverage * 100
        Else
            Return Double.NaN
        End If
    End Function

    Private Function CalcYearAveragePerFeature() As Dictionary(Of Feature, FeatureInfo)
        Dim infos As Dictionary(Of Feature, FeatureInfo) = New Dictionary(Of Feature, FeatureInfo)()

        For Each _vote As Vote In VoteList
            Dim _featureInfo As FeatureInfo = Nothing
            If infos.ContainsKey(_vote.Feature) Then
                _featureInfo = infos(_vote.Feature)
            Else
                _featureInfo = New FeatureInfo
                infos(_vote.Feature) = _featureInfo
            End If
            _featureInfo.AddValue(_vote.Priority)
        Next

        Return infos
    End Function

    Public Class FeatureInfo
        Public Sub AddValue(ByVal value As Integer)
            _prioritySum += value
            _valCount += 1
        End Sub
        Private _prioritySum As Integer
        Public ReadOnly Property PrioritySum() As Integer
            Get
                Return _prioritySum
            End Get
        End Property
        Private _valCount As Integer
        Public ReadOnly Property ValCount() As Integer
            Get
                Return _valCount
            End Get
        End Property
        Public ReadOnly Property Average() As Double
            Get
                Return _prioritySum / _valCount
            End Get
        End Property
    End Class

    Public Overrides ReadOnly Property IsReadOnly() As Boolean
        Get
            Return True
        End Get
    End Property

    Public Overrides ReadOnly Property PropertyType() As System.Type
        Get
            Return GetType(Double)
        End Get
    End Property

    Public Overrides Sub ResetValue(ByVal component As Object)

    End Sub

    Public Overrides Sub SetValue(ByVal component As Object, ByVal value As Object)

    End Sub

    Public Overrides Function ShouldSerializeValue(ByVal component As Object) As Boolean
        Return False
    End Function
End Class
