Imports System.ComponentModel
Imports Sturm.CollectionWrapping

Public Class FeatureVotingForm
    Public Sub New()
        InitializeComponent()

        CreateCustomGridStyles()
        InitGrid(InitDemoData())
    End Sub

    Private Sub InitGrid(ByVal _dataSource As Object)
        FeatureView.AutoGenerateColumns = False

        For Each _propertyDescriptor As PropertyDescriptor In CType(_dataSource, ITypedList).GetItemProperties(Nothing)
            Dim newColumn As DataGridViewColumn = Nothing

            If TypeOf _propertyDescriptor Is VoteValuePropertyDescriptor Then
                newColumn = New VoteValueColumn
            Else
                newColumn = New DataGridViewTextBoxColumn
            End If

            newColumn.DataPropertyName = _propertyDescriptor.Name
            newColumn.HeaderText = _propertyDescriptor.DisplayName
            FeatureView.Columns.Add(newColumn)
        Next

        FeatureView.DataSource = _dataSource
    End Sub

    Private Function InitDemoData() As Object
        Dim featureList As BindingList(Of Feature) = New BindingList(Of Feature)

        featureList.Add(New Feature("Synthesize out-of-the-box supply-chains"))
        featureList.Add(New Feature("Benchmark magnetic relationships"))
        featureList.Add(New Feature("Seize wireless web services"))
        featureList.Add(New Feature("Aggregate customized relationships"))
        featureList.Add(New Feature("Empower B2B platforms"))
        featureList.Add(New Feature("Incubate integrated bandwidth"))
        featureList.Add(New Feature("Syndicate vertical mindshare"))
        featureList.Add(New Feature("Implement world-class e-commerce"))
        featureList.Add(New Feature("Unleash impactful partnerships"))
        featureList.Add(New Feature("Engage ubiquitous paradigms"))
        featureList.Add(New SummaryFeature())

        Dim voteList As BindingList(Of Vote) = New BindingList(Of Vote)

        Dim _random As Random = New Random
        For _year As Integer = 2003 To 2007
            For Each _feature As Feature In featureList
                voteList.Add(New Vote(_feature, _year, _random.Next(15)))
            Next
        Next

        Return TypedListWrapper.CreateWrapper(featureList, CalculatePropertyDescriptors(featureList, voteList))
    End Function

    Private Function CalculatePropertyDescriptors(ByVal _featureList As BindingList(Of Feature), ByVal _voteList As BindingList(Of Vote)) As PropertyDescriptorCollection
        Dim origProperties As PropertyDescriptorCollection = TypeDescriptor.GetProperties(GetType(Feature))

        Dim properties As List(Of PropertyDescriptor) = New List(Of PropertyDescriptor)
        For Each _propertyDescriptor As PropertyDescriptor In origProperties
            properties.Add(_propertyDescriptor)
        Next

        Dim handledYears As List(Of Integer) = New List(Of Integer)

        For Each _vote As Vote In _voteList
            If Not handledYears.Contains(_vote.Year) Then
                properties.Add(New VoteValuePropertyDescriptor(_featureList, _voteList, _vote.Year))
                handledYears.Add(_vote.Year)
            End If
        Next

        properties.Add(New FeatureSummaryPropertyDescriptor(_featureList, _voteList))

        Return New PropertyDescriptorCollection(properties.ToArray)
    End Function

    Dim rowHeaderStyle As DataGridViewCellStyle
    Dim summaryColumnStyle As DataGridViewCellStyle
    Dim summaryRowStyle As DataGridViewCellStyle
    Dim crossHairStyle As DataGridViewCellStyle

    Private Sub CreateCustomGridStyles()
        rowHeaderStyle = New DataGridViewCellStyle(FeatureView.ColumnHeadersDefaultCellStyle)
        rowHeaderStyle.SelectionBackColor = rowHeaderStyle.BackColor
        rowHeaderStyle.SelectionForeColor = rowHeaderStyle.ForeColor

        summaryColumnStyle = New DataGridViewCellStyle(FeatureView.DefaultCellStyle)
        summaryColumnStyle.BackColor = rowHeaderStyle.BackColor
        summaryColumnStyle.SelectionBackColor = summaryColumnStyle.BackColor
        summaryColumnStyle.SelectionForeColor = summaryColumnStyle.ForeColor
        summaryColumnStyle.Font = New Font(summaryColumnStyle.Font, FontStyle.Bold)

        summaryRowStyle = New DataGridViewCellStyle(summaryColumnStyle)

        summaryColumnStyle.Format = "N2"

        crossHairStyle = New DataGridViewCellStyle(FeatureView.DefaultCellStyle)
        crossHairStyle.BackColor = SystemColors.Info
    End Sub


    Private Sub FeatureView_CellFormatting(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DataGridViewCellFormattingEventArgs) Handles FeatureView.CellFormatting
        If FeatureView.Columns(e.ColumnIndex).DataPropertyName = "Name" Then
            e.CellStyle = rowHeaderStyle
        ElseIf FeatureView.Columns(e.ColumnIndex).DataPropertyName = FeatureSummaryPropertyDescriptor.STR_PropertyName Then
            e.CellStyle = summaryColumnStyle
        ElseIf Attribute.IsDefined(CType(FeatureView.DataSource, IList(Of Feature))(e.RowIndex).GetType, GetType(SummaryFeatureAttribute)) Then
            Dim intVal As Integer = CInt(e.Value)
            Dim cellStyle As DataGridViewCellStyle = Nothing
            If intVal <> 100 Then
                cellStyle = New DataGridViewCellStyle(summaryRowStyle)
                cellStyle.ForeColor = IIf(intVal < 100, Color.Green, Color.Red)
            Else
                cellStyle = summaryRowStyle
            End If
            e.CellStyle = cellStyle
        ElseIf e.ColumnIndex = FeatureView.CurrentCellAddress.X OrElse e.RowIndex = FeatureView.CurrentCellAddress.Y Then
            e.CellStyle = crossHairStyle
        End If
    End Sub

    Private oldRow As Nullable(Of Integer) = Nothing
    Private oldColumn As Nullable(Of Integer) = Nothing

    Private Sub FeatureView_CellLeave(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DataGridViewCellEventArgs) Handles FeatureView.CellLeave
        oldRow = e.RowIndex
        oldColumn = e.ColumnIndex
    End Sub

    Private Sub FeatureView_CellEnter(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DataGridViewCellEventArgs) Handles FeatureView.CellEnter
        If oldColumn.HasValue Then
            FeatureView.InvalidateColumn(oldColumn.Value)
            oldColumn = Nothing
        End If
        If oldRow.HasValue Then
            FeatureView.InvalidateRow(oldRow.Value)
            oldRow = Nothing
        End If

        FeatureView.InvalidateRow(e.RowIndex)
        FeatureView.InvalidateColumn(e.ColumnIndex)
    End Sub

End Class
