Imports System.ComponentModel


Public Class VoteValuePropertyDescriptor
    Inherits PropertyDescriptor

    Public Sub New(ByVal featureList As BindingList(Of Feature), ByVal voteList As BindingList(Of Vote), ByVal year As Integer)
        MyBase.New(year.ToString, Nothing)
        _featureList = featureList
        _voteList = voteList
        AddHandler _voteList.ListChanged, New ListChangedEventHandler(AddressOf voteList_ListChanged)
        _year = year
    End Sub

    Private _featureList As BindingList(Of Feature)
    Public ReadOnly Property FeatureList() As BindingList(Of Feature)
        Get
            Return _featureList
        End Get
    End Property

    Private _voteList As BindingList(Of Vote)
    Public ReadOnly Property VoteList() As BindingList(Of Vote)
        Get
            Return _voteList
        End Get
    End Property

    Private _year As Integer
    Public ReadOnly Property Year() As Integer
        Get
            Return _year
        End Get
    End Property

    Private Sub voteList_ListChanged(ByVal sender As Object, ByVal e As ListChangedEventArgs)
        If e.ListChangedType = ListChangedType.ItemChanged Then
            PushChangeToSummaryFeatures()
        End If
    End Sub

    Private Sub PushChangeToSummaryFeatures()
        For Each _feature As Feature In _featureList
            If IsSummaryFeature(_feature) Then
                _feature.RaisePropertyChanged(Year.ToString)
            End If
        Next
    End Sub


    Public Overrides Function CanResetValue(ByVal component As Object) As Boolean
        Return False
    End Function

    Public Overrides ReadOnly Property ComponentType() As System.Type
        Get
            Return GetType(Feature)
        End Get
    End Property

    Public Overrides Function GetValue(ByVal component As Object) As Object
        Dim _feature As Feature = CType(component, Feature)
        If issummaryfeature(_feature) Then
            Return calcprioritysumforyear
        Else
            Dim _vote As Vote = getvotefor(_voteList, _feature, _year)
            Return _vote.Priority
        End If
    End Function

    Private Function CalcPrioritySumForYear(ByVal _voteList As BindingList(Of Vote), ByVal _year As Integer) As Integer
        Dim sum As Integer = 0
        For Each _vote As Vote In _voteList
            If _vote.Year = _year Then
                sum += _vote.Priority
            End If
        Next
        Return sum
    End Function

    Public Function CalcPrioritySumForYear() As Integer
        Return CalcPrioritySumForYear(_voteList, _year)
    End Function

    Private Function IsSummaryFeature(ByVal _feature As Feature) As Boolean
        Dim featureType As Type = _feature.GetType
        Return Attribute.IsDefined(featureType, GetType(SummaryFeatureAttribute))
    End Function

    Private Function GetVoteFor(ByVal _voteList As IList(Of Vote), ByVal _feature As Feature, ByVal _year As Integer) As Vote
        For Each _vote As Vote In _voteList
            If _vote.Feature Is _feature AndAlso _vote.Year = _year Then
                Return _vote
            End If
        Next
        Dim newVote As Vote = New Vote(_feature, _year, 0)
        _voteList.Add(newVote)
        Return newVote
    End Function

    Public Overrides ReadOnly Property IsReadOnly() As Boolean
        Get
            Return False
        End Get
    End Property

    Public Overrides ReadOnly Property PropertyType() As System.Type
        Get
            Return GetType(Integer)
        End Get
    End Property

    Public Overrides Sub ResetValue(ByVal component As Object)

    End Sub

    Public Overrides Sub SetValue(ByVal component As Object, ByVal value As Object)
        Dim _feature As Feature = CType(component, Feature)
        If Not IsSummaryFeature(_feature) Then
            Dim _vote As Vote = GetVoteFor(_voteList, _feature, _year)
            _vote.Priority = CInt(value)
        End If
    End Sub

    Public Overrides Function ShouldSerializeValue(ByVal component As Object) As Boolean
        Return False
    End Function
End Class