Imports System.ComponentModel

Public Class TypedListWrapper
    Implements ITypedList

#Region "Creation"
#Region "Constructors"
    Public Sub New()
    End Sub

    Public Sub New(ByVal propertyDescriptorCollection As PropertyDescriptorCollection)
        _propertyDescriptorCollection = propertyDescriptorCollection
    End Sub
#End Region

#Region "Typed shared creation helpers"
    Public Shared Function CreateWrapper(Of T)(ByVal wrappedList As BindingList(Of T)) As BindingList(Of T)
        Return CreateWrapper(wrappedList, Nothing)
    End Function

    Public Shared Function CreateWrapper(Of T)(ByVal wrappedList As BindingList(Of T), ByVal _propertyDescriptorCollection As PropertyDescriptorCollection) As BindingList(Of T)
        Return New TypedListWrapperBindingListT(Of T)(wrappedList, _propertyDescriptorCollection)
    End Function

    Public Shared Function CreateWrapper(ByVal wrappedList As IBindingList) As IBindingList
        Return CreateWrapper(wrappedList, Nothing)
    End Function

    Public Shared Function CreateWrapper(ByVal wrappedList As IBindingList, ByVal _propertyDescriptorCollection As PropertyDescriptorCollection) As IBindingList
        Return New TypedListWrapperIBindingList(wrappedList, _propertyDescriptorCollection)
    End Function

    Public Shared Function CreateWrapper(Of T)(ByVal wrappedList As IList(Of T)) As IList(Of T)
        Return CreateWrapper(wrappedList, Nothing)
    End Function

    Public Shared Function CreateWrapper(Of T)(ByVal wrappedList As IList(Of T), ByVal _propertyDescriptorCollection As PropertyDescriptorCollection) As IList(Of T)
        Return New TypedListWrapperIListT(Of T)(wrappedList, _propertyDescriptorCollection)
    End Function

    Public Shared Function CreateWrapper(ByVal wrappedList As IList) As IList
        Return CreateWrapper(wrappedList, Nothing)
    End Function

    Public Shared Function CreateWrapper(ByVal wrappedList As IList, ByVal _propertyDescriptorCollection As PropertyDescriptorCollection) As IList
        Return New TypedListWrapperIList(wrappedList, _propertyDescriptorCollection)
    End Function

#End Region

#Region "Untyped shared creation helpers"
    Public Shared Function CreateWrapper(ByVal wrappedList As Object) As Object
        Return CreateWrapper(wrappedList, Nothing)
    End Function

    Public Shared Function CreateWrapper(ByVal wrappedList As Object, ByVal _propertyDescriptorCollection As PropertyDescriptorCollection) As Object
        Dim listType As Type = wrappedList.GetType
        Dim openWrapperType As Type = Nothing
        Dim wrapperTypeArguments As Type() = Nothing

        If DerivesFromBindingListT(listType) Then
            openWrapperType = GetType(TypedListWrapperBindingListT(Of ))
            wrapperTypeArguments = GetBindingListTypeArguments(listType)
        ElseIf ImplementsInterface(listType, GetType(IBindingList)) Then
            openWrapperType = GetType(TypedListWrapperIBindingList)
        ElseIf ImplementsInterface(listType, GetType(IList(Of ))) Then
            openWrapperType = GetType(TypedListWrapperIListT(Of ))
            wrapperTypeArguments = GetIListTTypeArguments(listType)
        ElseIf ImplementsInterface(listType, GetType(IList)) Then
            openWrapperType = GetType(TypedListWrapperIList)
        End If

        If openWrapperType IsNot Nothing Then
            Dim finalWrapperType As Type = openWrapperType
            If openWrapperType.ContainsGenericParameters Then
                finalWrapperType = openWrapperType.MakeGenericType(wrapperTypeArguments)
            End If
            Return Activator.CreateInstance(finalWrapperType, New Object() {wrappedList, _propertyDescriptorCollection})
        End If

        Return Nothing
    End Function

#End Region

#Region "Reflection helpers"
    Private Shared Function DerivesFromBindingListT(ByVal _type As Type) As Boolean
        If _type Is GetType(Object) Then
            Return False
        End If
        If IsBindingListT(_type) Then
            Return True
        End If
        Return DerivesFromBindingListT(_type.BaseType)
    End Function

    Private Shared Function GetBindingListTypeArguments(ByVal _type As Type) As Type()
        If _type Is GetType(Object) Then
            Return Nothing
        End If
        If IsBindingListT(_type) Then
            Return _type.GetGenericArguments
        End If
        Return GetBindingListTypeArguments(_type.BaseType)
    End Function

    Private Shared Function GetIListTTypeArguments(ByVal _type As Type) As Type()
        Dim interfaces As Type() = _type.GetInterfaces
        Dim ilistTType As Type = GetType(IList(Of ))

        For Each iface As Type In interfaces
            If iface.ContainsGenericParameters Then
                Dim openIface = iface.GetGenericTypeDefinition
                If openIface Is ilistTType Then
                    Return iface.GetGenericArguments
                End If
            End If
        Next

        Return Nothing
    End Function

    Private Shared Function IsOpenGenericType(ByVal _type As Type, ByVal _openGenericType As Type) As Boolean
        If _type.IsGenericType Then
            If _type.GetGenericTypeDefinition Is _openGenericType Then
                Return True
            End If
        End If
        Return False
    End Function

    Private Shared Function IsBindingListT(ByVal _type As Type) As Boolean
        Return IsOpenGenericType(_type, GetType(BindingList(Of )))
    End Function

    Private Shared Function ImplementsInterface(ByVal _type As Type, ByVal _interfaceType As Type) As Boolean
        Dim interfaces As Type() = _type.GetInterfaces

        For Each iface As Type In interfaces
            If iface Is _interfaceType OrElse _
            (_interfaceType.ContainsGenericParameters AndAlso _
            iface Is _interfaceType.GetGenericTypeDefinition) _
            Then
                Return True
            End If
        Next
        Return False
    End Function
#End Region
#End Region


    Private _propertyDescriptorCollection As PropertyDescriptorCollection
    Public Property PropertyDescriptorCollection() As PropertyDescriptorCollection
        Get
            Return _propertyDescriptorCollection
        End Get
        Set(ByVal Value As PropertyDescriptorCollection)
            _propertyDescriptorCollection = Value
        End Set
    End Property

    Private _listName As String = "Default"
    Public Property ListName() As String
        Get
            Return _listName
        End Get
        Set(ByVal Value As String)
            _listName = Value
        End Set
    End Property

    Private Function ITypedList_GetItemProperties(ByVal listAccessors() As System.ComponentModel.PropertyDescriptor) As System.ComponentModel.PropertyDescriptorCollection Implements System.ComponentModel.ITypedList.GetItemProperties
        Return _propertyDescriptorCollection
    End Function

    Private Function ITypedList_GetListName(ByVal listAccessors() As System.ComponentModel.PropertyDescriptor) As String Implements System.ComponentModel.ITypedList.GetListName
        Return _listName
    End Function
End Class
